package info.textgrid.middleware;

import java.io.StringReader;
import java.text.ParseException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.apache.commons.logging.LogFactory;
import org.classicmayan.tools.ClassicMayanMetsMods;
import org.elasticsearch.action.get.GetRequestBuilder;
import org.elasticsearch.action.get.GetResponse;
import info.textgrid.middleware.oaipmh.GetRecordType;
import info.textgrid.middleware.oaipmh.HeaderType;
import info.textgrid.middleware.oaipmh.MetadataType;
import info.textgrid.middleware.oaipmh.RecordType;
import info.textgrid.middleware.oaipmh.StatusType;

/**
 * 
 * @author Maximilian Brodhun, SUB Göttingen
 * @since 2019-03-08
 * @version 2019-03-12
 *
 */

public class RecordDelivererIDIOM extends RecordDelivererAbstract {

  private String dateOfObjectCreation;
  private String dateOfLastOblectModification;
  private OAI_ESClient oaiEsClient;

  
  RecordType record = new RecordType();
  //ClassicMayanMetsMods metsmods;

  private static org.apache.commons.logging.Log log = LogFactory.getLog(RecordDelivererIDIOM.class);

  /**
   * @param textgrid
   * @param dariah
   */
  public RecordDelivererIDIOM(boolean textgrid, boolean dariah) {
    super(textgrid, dariah);
  }

  /*
   * (non-Javadoc)
   * 
   * @see info.textgrid.middleware.RecordDelivererInterface#getRecordById(java.lang.String)
   */

  /**
   * ElasticSearch request in non-public index to get the datestamps for the object creation and
   * last modification date in TextGrid-Repo
   * 
   * @param idInDatabase
   */

  public void setDatestamps(String idInDatabase) {
    String changedId = idInDatabase;

    if (idInDatabase.startsWith("textgrid:")) {
      changedId = idInDatabase.substring("textgrid:".length());
    }

    if (!idInDatabase.contains(".0")) {
    	
      //changedId = changedId.substring(0,changedId.length() - 1) + "0";
      changedId = changedId + ".0";

    }
    if(changedId.contains(".1")) {
  	  changedId = changedId.replace(".1", "");
    }
    
    
    System.out.println("id in elasticsearch is: " + changedId);
    GetRequestBuilder recordById = OAI_ESClient.getOaiESClient()
        .prepareGet()
        .setIndex("textgrid-nonpublic")
        .setType(OAI_ESClient.getEsType())
        .setFields(TGConstants.CREATED, TGConstants.MODIFIED_FIELD)
        .setId(changedId);

    GetResponse objectInDatabase = recordById.execute().actionGet();

    
    this.dateOfObjectCreation =
        objectInDatabase.getField(TGConstants.CREATED).getValue().toString();
    
    this.dateOfLastOblectModification =
        objectInDatabase.getField(TGConstants.MODIFIED_FIELD).getValue().toString();
  }

  /**
   * building the record XML object for the OAI-PMH response
   */

  public GetRecordType getRecordById(String id) {
	ClassicMayanMetsMods metsmods;
    GetRecordType grt = new GetRecordType();

    setDatestamps(id);
    String textgridBaseURI_FromID;
    if (!id.startsWith("textgrid:")) {
      textgridBaseURI_FromID = "textgrid:" + id.replace(".0", "");
    } else {
      textgridBaseURI_FromID = id;
    }
    log.info("ID for generating the METS/MODS file is: " + textgridBaseURI_FromID);
    metsmods = new ClassicMayanMetsMods(
        textgridBaseURI_FromID,
        this.dateOfObjectCreation,
        this.dateOfLastOblectModification);
    this.record.setMetadata(idiomMets(metsmods));
    try {
      if (!id.startsWith("textgrid:")) {
        setRecordHeader(this.dateOfObjectCreation, "textgrid:" + id);
      } else {
        setRecordHeader(this.dateOfObjectCreation, id);
      }
    } catch (ParseException e) {
      e.printStackTrace();
      log.error(e.toString());
    }

    grt.setRecord(this.record);

    return grt;
  }

  // **
  // PRIVATE METHODS
  // **

  /**
   * Getting the Mets/Mods object for the ClassicMayan artefact and put them into metadata element
   * of the OAI-PMH response
   * 
   * @return metadata XML-Elment
   */

  private MetadataType idiomMets(ClassicMayanMetsMods metsmods) {

	MetadataType metadataMets = new MetadataType();
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    DocumentBuilder builder;

    try {
      builder = factory.newDocumentBuilder();
      Document doc = builder.parse(new InputSource(new StringReader(metsmods.getMets())));
      metadataMets.setAny(doc.getDocumentElement());
    } catch (Exception e) {
      e.printStackTrace();
    }

    return metadataMets;
  }



  /**
   * @param dateOfCreation
   * @param identifier
   * @return
   * @throws ParseException
   */
  private void setRecordHeader(String dateOfCreation, String identifier) throws ParseException {

    HeaderType recordHeader = new HeaderType();
    recordHeader.setIdentifier(identifier);
    recordHeader.setDatestamp(OAIPMHUtilities.datestampAsString(dateOfCreation));
    this.record.setHeader(recordHeader);
    
    //recordHeader.setStatus(StatusType.DELETED);
    // return recordHeader;
  }

  public OAI_ESClient getOaiEsClient() {
    return this.oaiEsClient;
  }

  public void setOaiEsClient(OAI_ESClient oaiEsClient) {
    this.oaiEsClient = oaiEsClient;
  }

}
