package info.textgrid.middleware;

import java.util.Hashtable;
import java.util.Map;
import org.apache.commons.logging.LogFactory;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.common.unit.TimeValue;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;

import info.textgrid.middleware.oaipmh.HeaderType;
import info.textgrid.middleware.oaipmh.ListRecordsType;
import info.textgrid.middleware.oaipmh.MetadataType;
import info.textgrid.middleware.oaipmh.RecordType;
import info.textgrid.middleware.oaipmh.ResumptionTokenType;

/**
 *
 */
public class RecordListDelivererIDIOM extends RecordListDelivererAbstract {

  private static org.apache.commons.logging.Log log = LogFactory.getLog(OAIPMHImpl.class);
  static Map<String, Integer> cursorCollector = new Hashtable<String, Integer>();

  private OAI_ESClient oaiEsClient;
  private boolean foundItems;

  /**
   * @param textgrid
   * @param dariah
   */
  public RecordListDelivererIDIOM(boolean textgrid, boolean dariah) {
	
    super(textgrid, dariah);
  }

  /**
   * @param recordList
   * @param set
   * @param headerIdentifier
   * @return
   */
  public RecordType buildRecord(ListRecordsType recordList, String set, String headerIdentifier) {

    MetadataType metadata = new MetadataType();
    RecordType record = new RecordType();

    record.setMetadata(metadata);
    recordList.getRecord().add(record);

    return null;
  }

  /*
   * (non-Javadoc)
   * 
   * @see info.textgrid.middleware.RecordListDelivererInterface#getRecords(java.lang.String,
   * java.lang.String, java.lang.String, java.lang.String)
   */

  @Override
  public ListRecordsType getRecords(final String from, final String to, final String set,
	      final String resumptionToken) {
      System.out.println("BUFF");
	  ListRecordsType recordList = new ListRecordsType();
	  
	  QueryBuilder recordFilterForClassicMayan;
	  QueryBuilder rangeQuery;
	  //Queries queries = new Queries();
	  //List<String> artefactURIs = queries.getArtefactList();
	  
	  rangeQuery = QueryBuilders.rangeQuery("lastModified").from(from).to(to);
	  recordFilterForClassicMayan = QueryBuilders.boolQuery().must(rangeQuery)
			  .must(QueryBuilders.matchPhraseQuery("project.id", "TGPR-0e926f53-1aba-d415-ecf6-539edcd8a318"))
			  .must(QueryBuilders.matchPhraseQuery("format", "text/tg.inputform+rdf+xml"))
			  .must(QueryBuilders.matchPhraseQuery("notes", "ARTEFACT"));
	  
	  SearchResponse scrollResp;
	  
	  if (resumptionToken == null) {
		  
	      scrollResp = OAI_ESClient.getOaiESClient()
	              .prepareSearch("textgrid-nonpublic")
	              .setScroll(TimeValue.timeValueHours(24L))
	              .setTypes(OAI_ESClient.getEsType())
	              .addFields("lastModified", "textgridUri", "project.id")
	              .setQuery(recordFilterForClassicMayan)
	              .setSize(30)
	              .execute()
	              .actionGet();
	  }else {
	      scrollResp = OAI_ESClient.getOaiESClient().prepareSearchScroll(resumptionToken)
	              .setScroll(TimeValue.timeValueHours(24L)).execute().actionGet();
	  }
	  
	  String scrollID = scrollResp.getScrollId();
	  
	  long completeListSize = scrollResp.getHits().totalHits();
	  //long completeListSize = 45;
	    //System.out.println(completeListSize);
	    //System.out.println(completeListSize + " || " + artefactURIs.size());
	    if (completeListSize > 0) {
	        setFoundItems(true);
	        int i = 0;
	        //System.out.println("inside ");
	        for (SearchHit hit : scrollResp.getHits().getHits()) {	          
	          i++;
	          String textgridURI = hit.getFields().get("textgridUri").getValue().toString().replace(".0", "");
	          System.out.println("Processing: " + textgridURI);
        	  RecordDelivererIDIOM idiomRecord = new RecordDelivererIDIOM(true, false);
        	  recordList.getRecord().add(idiomRecord.getRecordById(textgridURI).getRecord());
	        }

		   // Check the need for a resumption token!
		   ResumptionTokenType resTokenForResponse = OAIPMHUtilities.getResumptionToken(
		          completeListSize, resumptionToken, cursorCollector, scrollID, 30, i);
		   if (resTokenForResponse != null) {
			   recordList.setResumptionToken(resTokenForResponse);
		   }
	    }else {
	      setFoundItems(false);
	    }
	    
	 return recordList;
  }
  
  /*
   * (non-Javadoc)
   * 
   * @see info.textgrid.middleware.RecordListDelivererInterface#setHeader(java.lang.String,
   * java.lang.String)
   */
  @Override
  public HeaderType setHeader(String set, String headerIdentifier) {
    return null;
  }

  /**
   * @return
   */
  public OAI_ESClient getOaiEsClient() {
    return this.oaiEsClient;
  }

  /**
   * @param oaiEsClient
   */
  public void setOaiEsClient(OAI_ESClient oaiEsClient) {
    this.oaiEsClient = oaiEsClient;
  }

public boolean isFoundItems() {
	return foundItems;
}

public void setFoundItems(boolean foundItems) {
	this.foundItems = foundItems;
}

}
